/*----------------------------------------------------------------------
DPF2XML - Roberto Prieto
Copyright (C) 2007-2016 MegaStorm Systems
contact@megastormsystems.com - http://www.megastormsystems.com

This software is provided 'as-is', without any express or implied
warranty.  In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not
claim that you wrote the original software. If you use this software
in a product, an acknowledgment in the product documentation would be
appreciated but is not required.
2. Altered source versions must be plainly marked as such, and must not be
misrepresented as being the original software.
3. This notice may not be removed or altered from any source distribution.

------------------------------------------------------------------------

DPF2XML

Export DPF resources to a XML file using MegaStorm Interchange XML v1.0 format (MIXv1)
Import a MIXv1 format into a DPF

Changelog:
- v2.0 Jul-2016: added sound, music, files and buttons import/export.
                 Improved error checking and reporting.
- v1.2 Jun-2007: first public version.

------------------------------------------------------------------------ */

#include <CRM32Pro.h>
#include <time.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <direct.h>
#include <io.h>

#define DIR_SLASH "/"
#define DPF2XML_VERSION "2.0"

int DPF2XML(char* szInput, char* szOutput);
int widgetAddState(int idDPF, int idXML, char* pName, char* pState, char* pSource);
int widgetLoadState(int idDPF, int idXML, char* pName, char* pState);
int XML2DPF(char* szInput, char* szOutput);
int RemoveBlock(int idDPF, char *type, char *name);

int main(int argc, char *argv[])
{
	char szExtension[32];
    char* szOutput = NULL;

	// Parameter parser (we only need one)
	printf("---------------------------------------------------------\n");
	printf("DPF2XML v%s - DPF resources export to XML and vice versa\n", DPF2XML_VERSION);
	printf("   MegaStorm Systems (c) 2007-2016 - CRM32Pro SDK\n");
	printf("---------------------------------------------------------\n\n");
    puts(" Export DPF resources to a XML file using MegaStorm Interchange");
    puts(" XML v1.0 format(MIXv1) or import a MIXv1 format into a DPF");
    if(argc < 2)
	{
        puts(" Usage: DPF2XML.exe input.[xml|dpf] [output.dpf|output.xml]\n");
		return 0;
	}
	_splitpath(argv[1], NULL, NULL, NULL, szExtension);
	strcpy(szExtension, _strlwr((char*)szExtension));
    if(argc > 2) szOutput = argv[2];

	// CRM32Pro init at minimum level
	CRM32Pro.Init(0);

	// Depending on the parameter extension, we proceed with import or export the DPF
	if(!strcmp(szExtension, ".dpf"))
	{
		printf(" - Processing blocks...\n");
		if(!DPF2XML(argv[1], szOutput)) printf("\n - Error! the DPF was not exported.\n");
		else printf("\n - DPF exported successfully to a XML + gfx resources.\n");
	}
	else if(!strcmp(szExtension, ".xml"))
	{
		printf(" - Processing blocks...\n");
		if(!XML2DPF(argv[1], szOutput)) printf("\n - Error! the XML was not imported.\n");
		else printf("\n - XML imported successfully into a new DPF.\n");
	}
	else
	{
		printf(" - Error: input file it is not supported. Try with a DPF or XML (MIXv1 format) file\n");
	}

	// End of execution 
	CRM32Pro.Quit();
	return 0;
}

// Export resources from a DPF to XML and a folder with all BMP/PNG
// Supports images, tiles, fonts, sprites, music, sounds, files and buttons
int DPF2XML(char* szInput, char* szOutput)
{
	SDL_Surface* pSurface;
	CRM32Pro_CFont* pFont;
	CRM32Pro_CSprite* pSprite;
	CRM32Pro_CTile* pTile;
	DPF_BlockData dpfBlock, myBlock;
    sAudioType AT;
	int status = 0, idXML = 0, retvalue = 0, idDPF = 0, ret = 0, iExport = 0;
    int totalBlocks = 0;
	int fontBlocks = 0, spriteBlocks = 0, imageBlocks = 0, tileBlocks = 0;
    int musicBlocks = 0, soundBlocks = 0, fileBlocks = 0, widgetBlocks = 0;
	char szFileName[255], szFileXML[255], szSurface[255], szBlockName[32], *pTmp;
	long ltime;
	const struct tm   *localt;

	// Open DPF and init
	idDPF = IStuffDPF->Open(szInput);
	if(idDPF == 0)
	{
		printf(" - Error: could not open the given DPF.\n");
		return ret;
	}
	totalBlocks = IStuffDPF->GetNumBlocks(idDPF);
	if(totalBlocks == 0)
	{
		printf(" - Error: could not access to the given DPF. Is it password protected?\n");
		IStuffDPF->Close(idDPF);
		return ret;
	}
	totalBlocks = totalBlocks - IStuffDPF->GetNumDelBlocks(idDPF);

	// Read the first block
	status = IStuffDPF->FindFirst(idDPF, &dpfBlock);

	// If there is a block: create the folder for the BMP/PNG and the XML
	if(status == 1)
	{
        // Get output name if possible or use the source one
        if(szOutput == NULL)
        {
            strncpy(szFileName, szInput, strlen(szInput) - 4);            
            szFileName[strlen(szInput) - 4] = '\0';
        }
        else
        {
            if(strlen(szOutput) < 5)
            {
                printf(" - Error: output file requires a valid extension.\n");
                IStuffDPF->Close(idDPF);
                return ret;
            }
            strncpy(szFileName, szOutput, strlen(szOutput) - 4);
            szFileName[strlen(szOutput) - 4] = '\0';
        }
        strcpy(szFileXML, szFileName);
        strcat(szFileXML, ".xml");
		
		idXML = CRM32Pro.XMLCreate("MIX");
		if(idXML == 0) status = -1;
        CRM32Pro.XMLCommentSet(idXML, "MegaStorm Interchange XML format - MegaStorm Systems (c)");
		retvalue = CRM32Pro.XMLAttributeSet(idXML, "version", "v1");
		if(idXML == 0) status = -1;
		retvalue = CRM32Pro.XMLAttributeSet(idXML, "source", szInput);
		if(retvalue == 0) status = -1;
		time(&ltime);
		localt = localtime(&ltime);
		retvalue = CRM32Pro.XMLAttributeSet(idXML, "date", asctime(localt));
		retvalue = CRM32Pro.XMLAttributeSet(idXML, "creator", "DPF2XML v2.0");
		_mkdir(szFileName);
	}

	// Loop through each block doing the export but widget which are a bit special
	while(status == 1)
	{
		// ----------------------------Exporting type = IMG----------------------------
		if(!strncmp("IMG", dpfBlock.type, 3))
		{
			// Load the surface
			pSurface = IImage->Load(idDPF, dpfBlock.name);
			// Write in the XML
			retvalue = CRM32Pro.XMLNodeCreate(idXML, "image"); // Create a new node and point to it.
			strcpy(szBlockName, "img_");
			strcat(szBlockName, dpfBlock.name);
			strcpy(szSurface, szFileName);
			strcat(szSurface, DIR_SLASH);
			strcat(szSurface, szBlockName);			
            retvalue = CRM32Pro.XMLAttributeSet(idXML, "name", dpfBlock.name);

			// Alpha per-pixel?
			if((pSurface->flags & SDL_SRCALPHA) && (pSurface->format->alpha == 255))
			{
				strcat(szSurface, ".png");
                retvalue = CRM32Pro.XMLAttributeSet(idXML, "source", szSurface);
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "alphablending", "per-pixel");
				IImage->SavePNG(szSurface, pSurface);
			}
			// Alpha per-surface
			else
			{
				strcat(szSurface, ".bmp");
                retvalue = CRM32Pro.XMLAttributeSet(idXML, "source", szSurface);
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "alphablending", "per-surface");
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "alphamod", (int)pSurface->format->alpha);
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "colorkey", (int)pSurface->format->colorkey);
				IImage->SaveBMP(szSurface, pSurface);
			}

			// Free the surface
			SDL_FreeSurface(pSurface);
			imageBlocks++;
			iExport++;
			CRM32Pro.XMLNodeParent(idXML); // Point to father node
		}

		// ----------------------------Exporting type = TILE | TILE2----------------------------
		else if(!strncmp("TILE", dpfBlock.type, 4))
		{
			pTile = new CRM32Pro_CTile;
			// Load the tile
			pTile->Load(idDPF, dpfBlock.name);
			pSurface = pTile->GetSurface();
			// Write in the XML
			retvalue = CRM32Pro.XMLNodeCreate(idXML, "tile"); // Create a new node and point to it.
			strcpy(szBlockName, "tile_");
			strcat(szBlockName, dpfBlock.name);
			strcpy(szSurface, szFileName);
			strcat(szSurface, DIR_SLASH);
			strcat(szSurface, szBlockName);
			retvalue = CRM32Pro.XMLAttributeSet(idXML, "name", dpfBlock.name);

			// Alpha per-surface?
			if(pTile->GetAlpha() != 256)
			{
				strcat(szSurface, ".bmp");
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "source", szSurface);
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "alphablending", "per-pSurface");
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "alphamod", (int)pSurface->format->alpha);
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "colorkey", (int)pSurface->format->colorkey);
				IImage->SaveBMP(szSurface, pSurface);
			}
			// Alpha per-pixel?
			else
			{
				strcat(szSurface, ".png");
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "source", szSurface);
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "alphablending", "per-pixel");
				IImage->SavePNG(szSurface, pSurface);
			}
			// Positions
			retvalue = CRM32Pro.XMLAttributeSet(idXML, "xposition", pTile->GetPosX());
			retvalue = CRM32Pro.XMLAttributeSet(idXML, "yposition", pTile->GetPosY());
			// Tileset
			int x, y;
			if(pTile->GetOffset(&x, &y))
			{
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "tileset", "enabled");
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "xoffset", x);
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "yoffset", y);
			}
			else retvalue = CRM32Pro.XMLAttributeSet(idXML, "tileset", "disabled");

			// Free the tile
			delete pTile;
			tileBlocks++;
			iExport++;
			CRM32Pro.XMLNodeParent(idXML); // Point to father node
		}

		// ----------------------------Exporting type = SPR | SPR2----------------------------
		else if(!strncmp("SPR", dpfBlock.type, 3))
		{
			pSprite = new CRM32Pro_CSprite;
			// Load the sprite
			pSprite->Load(idDPF, dpfBlock.name);
			pSurface = pSprite->GetSurface();

			// Write in the XML
			retvalue = CRM32Pro.XMLNodeCreate(idXML, "sprite"); // Create a new node and point to it
			strcpy(szBlockName, "spr_");
			strcat(szBlockName, dpfBlock.name);
			strcpy(szSurface, szFileName);
			strcat(szSurface, DIR_SLASH);
			strcat(szSurface, szBlockName);
			retvalue = CRM32Pro.XMLAttributeSet(idXML, "name", dpfBlock.name);

			// Alpha per-surface?
			if(pSprite->GetAlpha() != 256)
			{
				strcat(szSurface, ".bmp");
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "source", szSurface);
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "alphablending", "per-surface");
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "alphamod", (int)pSurface->format->alpha);
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "colorkey", (int)pSurface->format->colorkey);
				IImage->SaveBMP(szSurface, pSurface);
			}
			// Alpha per-pixel
			else
			{
				strcat(szSurface, ".png");
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "source", szSurface);
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "alphablending", "per-pixel");
				IImage->SavePNG(szSurface, pSurface);
			}
			// Offset
			int x = 0, y = 0;
			pSprite->GetOffset(&x, &y);
			retvalue = CRM32Pro.XMLAttributeSet(idXML, "xoffset", x);
			retvalue = CRM32Pro.XMLAttributeSet(idXML, "yoffset", y);

			// Loop through each animation
			for(int i = 0;i < pSurface->h / y;i++)
			{
				unsigned short type = 0, status = 0, frames = 0, startframe = 0, endframe = 0;
				signed short hotx = 0, hoty = 0;
				pSprite->GetAnimProperties(i + 1, &type, &status, &frames, &hotx, &hoty, &startframe, &endframe);
				retvalue = CRM32Pro.XMLNodeCreate(idXML, "animation"); // Create a new animation node and point to it
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "id", i);
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "type", type);
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "status", status);
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "frames", frames);
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "hotx", hotx);
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "hoty", hoty);
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "starframe", startframe);
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "endframe", endframe);
				// Loop through each frame
				for(int j = 0;j < frames;j++)
				{
					unsigned int ms = 0;
					retvalue = CRM32Pro.XMLNodeCreate(idXML, "frame"); // Create a new frame node and point to it
					retvalue = CRM32Pro.XMLAttributeSet(idXML, "id", j);
					pSprite->GetFrameProperties(i + 1, j + 1, &ms);
					retvalue = CRM32Pro.XMLAttributeSet(idXML, "ms", (int)ms);
					CRM32Pro.XMLNodeParent(idXML); // Point to father node
				}
				CRM32Pro.XMLNodeParent(idXML); // Point to father node

			}

			// Free the sprite
			delete pSprite;
			spriteBlocks++;
			iExport++;
			CRM32Pro.XMLNodeParent(idXML); // Point to father node
		}

		// ----------------------------Exporting type = FONT | FONT2----------------------------
		else if(!strncmp("FONT", dpfBlock.type, 4))
		{
			pFont = new CRM32Pro_CFont;
			// Load the font
			pFont->Load(idDPF, dpfBlock.name);
			pSurface = pFont->GetSurface();
			// Write in the XML
			retvalue = CRM32Pro.XMLNodeCreate(idXML, "font"); // Create a new node and point to it
			strcpy(szBlockName, "font_");
			strcat(szBlockName, dpfBlock.name);
			strcpy(szSurface, szFileName);
			strcat(szSurface, DIR_SLASH);
			strcat(szSurface, szBlockName);
			retvalue = CRM32Pro.XMLAttributeSet(idXML, "name", dpfBlock.name);

			// Alpha per-surface?
			if(pFont->GetAlpha() != 256)
			{
				strcat(szSurface, ".bmp");
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "source", szSurface);
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "alphablending", "per-surface");
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "alphamod", (int)pSurface->format->alpha);
                retvalue = CRM32Pro.XMLAttributeSet(idXML, "colorkey", (int)pSurface->format->colorkey);
				IImage->SaveBMP(szSurface, pSurface);
			}
			// Alpha per-pixel
			else
			{
				strcat(szSurface, ".png");
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "source", szSurface);
				retvalue = CRM32Pro.XMLAttributeSet(idXML, "alphablending", "per-pixel");
				IImage->SavePNG(szSurface, pSurface);
			}
			// Kerning
			retvalue = CRM32Pro.XMLAttributeSet(idXML, "kerning", pFont->GetKerning());

			// Free the font
			delete pFont;
			fontBlocks++;
			iExport++;
			CRM32Pro.XMLNodeParent(idXML); // Point to father node
		}

        // ----------------------------Exporting type = MUS----------------------------
        else if(!strncmp("MUS", dpfBlock.type, 3))
        {
            // Get music info                   
            ISoundFX->MusicInfo(idDPF, dpfBlock.name, &AT);
            // Write in the XML
            retvalue = CRM32Pro.XMLNodeCreate(idXML, "music"); // Create a new node and point to it
            strcpy(szBlockName, "mus_");
            strcat(szBlockName, dpfBlock.name);
            strcat(szBlockName, ".");
            strcat(szBlockName, AT.type);
            strcpy(szSurface, szFileName);
            strcat(szSurface, DIR_SLASH);
            strcat(szSurface, szBlockName);
            retvalue = CRM32Pro.XMLAttributeSet(idXML, "name", dpfBlock.name);
            retvalue = CRM32Pro.XMLAttributeSet(idXML, "source", szSurface);
            // Write the music file
            ISoundFX->Export(idDPF, dpfBlock.name, 1, szSurface);
            // Inc counter
            musicBlocks++;
            iExport++;
            CRM32Pro.XMLNodeParent(idXML); // Point to father node
        }

        // ----------------------------Exporting type = SND----------------------------
        else if(!strncmp("SND", dpfBlock.type, 3))
        {
            // Get sound info                   
            ISoundFX->SoundInfo(idDPF, dpfBlock.name, &AT);
            // Write in the XML
            retvalue = CRM32Pro.XMLNodeCreate(idXML, "sound"); // Create a new node and point to it
            strcpy(szBlockName, "snd_");
            strcat(szBlockName, dpfBlock.name);
            strcat(szBlockName, ".");
            strcat(szBlockName, AT.type);
            strcpy(szSurface, szFileName);
            strcat(szSurface, DIR_SLASH);
            strcat(szSurface, szBlockName);
            retvalue = CRM32Pro.XMLAttributeSet(idXML, "name", dpfBlock.name);
            retvalue = CRM32Pro.XMLAttributeSet(idXML, "source", szSurface);
            // Write the sound file
            ISoundFX->Export(idDPF, dpfBlock.name, 0, szSurface);
            // Inc counter
            soundBlocks++;
            iExport++;
            CRM32Pro.XMLNodeParent(idXML); // Point to father node
        }

        // ----------------------------Exporting type = FILE----------------------------
        else if(!strncmp("FILE", dpfBlock.type, 4))
        {            
            memset((void*)&myBlock, 0, sizeof(myBlock));
            // Write in the XML
            retvalue = CRM32Pro.XMLNodeCreate(idXML, "file"); // Create a new node and point to it
            strcpy(szBlockName, "file_");
            strcat(szBlockName, dpfBlock.name);
            strcpy(szSurface, szFileName);
            strcat(szSurface, DIR_SLASH);
            strcat(szSurface, szBlockName);
            retvalue = CRM32Pro.XMLAttributeSet(idXML, "name", dpfBlock.name);
            retvalue = CRM32Pro.XMLAttributeSet(idXML, "source", szSurface);            
            // Write the file
            IStuffDPF->LoadBlock(idDPF, "FILE", dpfBlock.name, &myBlock);
            int iFile = _open(szSurface, O_WRONLY | O_CREAT | O_TRUNC | O_BINARY, S_IWRITE);
            _write(iFile, myBlock.data, myBlock.size);
            _close(iFile);
            IStuffDPF->FreeBlock(&myBlock);
            // Write storemethod
            strcpy(szSurface, "raw");
            if(myBlock.storedmethod == 0) strcpy(szSurface, "compressed");
            else if(myBlock.storedmethod == 2) strcpy(szSurface, "crypted");
            retvalue = CRM32Pro.XMLAttributeSet(idXML, "storemethod", szSurface);
            // Inc counter
            fileBlocks++;
            iExport++;
            CRM32Pro.XMLNodeParent(idXML); // Point to father node
        }

		// Next block
		status = IStuffDPF->FindNext(idDPF, &dpfBlock);
		// Info
		printf(" - %d/%d: IMG %d - TIL %d - SPR %d - FNT %d - MUS %d - SND %d - FIL %d - GUI %d\r",
			iExport, totalBlocks, imageBlocks, tileBlocks, spriteBlocks, fontBlocks, musicBlocks, soundBlocks, fileBlocks, widgetBlocks);
	}

    // Special loop for exporting widgets as they are grouped on a single panel
    status = IStuffDPF->FindFirst(idDPF, &dpfBlock);
    while(status == 1)
    {
        // ----------------------------Exporting type = BUT----------------------------
        if(!strncmp("BUT", dpfBlock.type, 3))
        {
            IButtonNew myWidget;

            memset((void*)&myBlock, 0, sizeof(myBlock));
            // Create a panel containing all widgets
            if(widgetBlocks == 0)
            {
                retvalue = CRM32Pro.XMLNodeCreate(idXML, "panel"); // Create a new node and point to it.
                retvalue = CRM32Pro.XMLAttributeSet(idXML, "name", "default");
                retvalue = CRM32Pro.XMLAttributeSet(idXML, "xposition", 0); // Default value
                retvalue = CRM32Pro.XMLAttributeSet(idXML, "yposition", 0); // Default value
                retvalue = CRM32Pro.XMLAttributeSet(idXML, "width", 800); // Default value
                retvalue = CRM32Pro.XMLAttributeSet(idXML, "height", 600); // Default value
                retvalue = CRM32Pro.XMLAttributeSet(idXML, "alphamod", 0);  // Default value
            }
            // Write in the XML
            retvalue = CRM32Pro.XMLNodeCreate(idXML, "widget"); // Create a new node and point to it.
            strcpy(szBlockName, "wid_");
            strcat(szBlockName, dpfBlock.name);
            strcpy(szSurface, szFileName);
            strcat(szSurface, DIR_SLASH);
            strcat(szSurface, szBlockName);
            retvalue = CRM32Pro.XMLAttributeSet(idXML, "name", dpfBlock.name);

            // Load the widget for parsing its values
            IStuffDPF->LoadBlock(idDPF, "BUT", dpfBlock.name, &myBlock);
            memcpy(&myWidget, myBlock.data, sizeof(IButtonNew));
            IStuffDPF->FreeBlock(&myBlock);            
            pTmp = "unknown";
            switch(myWidget.type)
            {
            case B_TYPE_NORMAL:
            case B_TYPE_ICON:
            case B_TYPE_COMPLEX:
                pTmp = "button";
                break;
            case B_TYPE_SLIDER:
                pTmp = "hslider";
                break;
            case B_TYPE_PROGRESS:
                pTmp = "progressbar";
                break;
            case B_TYPE_CHECK:
                pTmp = "checkbox";
                break;
            case B_TYPE_INPUT:
                pTmp = "inputbox";
                break;
            }
            retvalue = CRM32Pro.XMLAttributeSet(idXML, "type", pTmp);
            retvalue = CRM32Pro.XMLAttributeSet(idXML, "id", widgetBlocks + 1); // In C32, the id was not "static" so we emulate it using a simple counter
            retvalue = CRM32Pro.XMLAttributeSet(idXML, "xposition", myWidget.posx);
            retvalue = CRM32Pro.XMLAttributeSet(idXML, "yposition", myWidget.posy);
            retvalue = CRM32Pro.XMLAttributeSet(idXML, "key", myWidget.key);
            retvalue = CRM32Pro.XMLAttributeSet(idXML, "selectboxRGB", (int)myWidget.color_select);
            retvalue = CRM32Pro.XMLAttributeSet(idXML, "value", myWidget.cvalue);
            retvalue = CRM32Pro.XMLAttributeSet(idXML, "range", myWidget.range);
            
            // Load tiles representing each state and parsing its values
            if(widgetAddState(idDPF, idXML, dpfBlock.name, "normal", szSurface)) iExport++;
            if(widgetAddState(idDPF, idXML, dpfBlock.name, "hovered", szSurface)) iExport++;
            if(widgetAddState(idDPF, idXML, dpfBlock.name, "pressed", szSurface)) iExport++;;
            if(widgetAddState(idDPF, idXML, dpfBlock.name, "deactivated", szSurface)) iExport++;;

            // Dirty-trick for re-positioning the internal DPF block pointer..
            IStuffDPF->LoadBlock(idDPF, "BUT", dpfBlock.name, &myBlock);
            IStuffDPF->FreeBlock(&myBlock);

            // Inc counter
            widgetBlocks++;
            iExport++;
            CRM32Pro.XMLNodeParent(idXML); // Point to father node
        }        

        // Next block
        status = IStuffDPF->FindNext(idDPF, &dpfBlock);
        
        // Info
        printf(" - %d/%d: IMG %d - TIL %d - SPR %d - FNT %d - MUS %d - SND %d - FIL %d - GUI %d\r",        
            iExport, totalBlocks, imageBlocks, tileBlocks, spriteBlocks, fontBlocks, musicBlocks, soundBlocks, fileBlocks, widgetBlocks);
    }

	// Close DPF
	IStuffDPF->Close(idDPF);

	// Was an error?
	if(status == -1)
	{
		printf(" - Error creating the output XML\n");
		return ret;
	}

	// Save to XML the real number of exported blocks
	retvalue = CRM32Pro.XMLNodePointTo(idXML, 1, "MIX");
	retvalue = CRM32Pro.XMLAttributeSet(idXML, "totalblocks", iExport);

	// Did we export anything? If not, remove unused files
	if(iExport == 0) _rmdir(szFileName);
	else CRM32Pro.XMLSave(idXML, szFileXML);
	CRM32Pro.XMLClose(idXML);

	// WARNING message: there were blocks not exported
	if(totalBlocks != iExport)
	{
		printf("\n - WARNING: existing DPF blocks(%d) does not match with exported ones(%d).", totalBlocks, iExport);
		printf("\n - Info: remember that button,sound,music and files blocks are not exported.\n");
	}

	// Return		
	return 1;
}

// Support function for exporting widget states
int widgetAddState(int idDPF, int idXML, char* pName, char* pState, char* pSource)
{
    CRM32Pro_CTile* pTile;
    SDL_Surface* pSurface;
    char pTmp[255];
    int iRet = 0;

    // Generate blockname
    strcpy(pTmp, pName);
    if(!strcmp(pState, "normal")) strcat(pTmp, "_n");
    else if(!strcmp(pState, "hovered")) strcat(pTmp, "_s");
    else if(!strcmp(pState, "pressed")) strcat(pTmp, "_p");
    else if(!strcmp(pState, "deactivated")) strcat(pTmp, "_o");

    // Try to load the tile
    pTile = new CRM32Pro_CTile;
    if(pTile->Load(idDPF, pTmp, "TILB2") != 0)
    {
        iRet = 1;
        strcpy(pTmp, pSource);
        strcat(pTmp, "_");
        strcat(pTmp, pState);
        pSurface = pTile->GetSurface();
        CRM32Pro.XMLNodeCreate(idXML, "image");
        CRM32Pro.XMLAttributeSet(idXML, "state", pState);
        // Alpha per-surface?
        if(pTile->GetAlpha() != 256)
        {
            strcat(pTmp, ".bmp");
            CRM32Pro.XMLAttributeSet(idXML, "source", pTmp);
            CRM32Pro.XMLAttributeSet(idXML, "alphablending", "per-surface");
            CRM32Pro.XMLAttributeSet(idXML, "alphamod", (int)pSurface->format->alpha);
            CRM32Pro.XMLAttributeSet(idXML, "colorkey", (int)pSurface->format->colorkey);
            IImage->SaveBMP(pTmp, pSurface);
        }
        // Alpha per-pixel?
        else
        {
            strcat(pTmp, ".png");
            CRM32Pro.XMLAttributeSet(idXML, "source", pTmp);
            CRM32Pro.XMLAttributeSet(idXML, "alphablending", "per-pixel");
            IImage->SavePNG(pTmp, pSurface);
        }
        CRM32Pro.XMLNodeParent(idXML);
    }
    delete pTile;
    return  iRet; 
}

// Import resources from a XML/folder to a given DPF
// .Support images, tiles, fonts and sprites
// musc, sound, files, buttons
int XML2DPF(char* szInput, char* szOutput)
{
	SDL_Surface* pSurface;
	CRM32Pro_CFont* pFont;
	CRM32Pro_CSprite* pSprite;
	CRM32Pro_CTile* pTile;
	int idXML = 0, retvalue = 0, iTmp = 0, iTmp2 = 0; // todo limpiar variables y dejar solo globales 
	int totalBlocks = 0;
	int fontBlocks = 0, spriteBlocks = 0, imageBlocks = 0, tileBlocks = 0;
    int musicBlocks = 0, soundBlocks = 0, fileBlocks = 0, widgetBlocks = 0;
	int idDPF, ret = 0;
	char szDPFFile[255], *pTmp, *pAux, *pNodeName; 
	int iImport = 0;
    int bFlagErr = -1;

	// Open XML
	idXML = CRM32Pro.XMLOpen(szInput);
	if(idXML == 0)
	{
		printf(" - Error: could not open the given XML.\n");
		return 0;
	}

	// Point to DPF_file node
	retvalue = CRM32Pro.XMLNodePointTo(idXML, 1, "MIX");
	if(retvalue == 0) { printf(" - Error parsing the XML, is it a MIX file?\n"); return 0; }
	// Get version
	retvalue = CRM32Pro.XMLAttributeGet(idXML, "version", &pTmp);
	if(strcmp(pTmp, "v1")) { printf(" - Only MIXv1 format is supported.\n"); return 0; }

    // Get output name if possible or use the source one
    if(szOutput == NULL) retvalue = CRM32Pro.XMLAttributeGet(idXML, "source", &pTmp);
    else pTmp = szOutput;
    strcpy(szDPFFile, pTmp);

	// Get attributes: totalblocks		
	retvalue = CRM32Pro.XMLAttributeGet(idXML, "totalblocks", &totalBlocks);

	// Open or create the DPF
	if((_access(szDPFFile, 0)) != -1) idDPF = IStuffDPF->Open(szDPFFile);
	else idDPF = IStuffDPF->Create(szDPFFile);
	if(idDPF == -1) { printf(" - Error: could not open/create the desired DPF('%s').\n", szDPFFile); CRM32Pro.XMLClose(idXML); return 0; }

	// Loop processing each block
	printf(" - Trying to generate '%s' with %d blocks\n", szDPFFile, totalBlocks);
	retvalue = CRM32Pro.XMLNodeChild(idXML);
	while(retvalue != 0)
	{
		// Get node name		
		pNodeName = CRM32Pro.XMLNodeGetName(idXML);

		// ----------------------------Importing images----------------------------
		if(!strncmp("image", pNodeName, 5))
		{
            bFlagErr = 1; // Assume error
            pTmp = "no name attribute";
            pSurface = NULL;

            // Get image name and blending mode
            if(CRM32Pro.XMLAttributeGet(idXML, "name", &pTmp) != 1) goto image_err;
            if(CRM32Pro.XMLAttributeGet(idXML, "alphablending", &pAux) != 1) goto image_err;
			
			// Alpha per-pixel?
            if(!strncmp("per-pixel", pAux, 9))
            {
                if(CRM32Pro.XMLAttributeGet(idXML, "source", &pAux) != 1) goto image_err;
                pSurface = IImage->LoadPNG(pAux);
            }
			// Alpha per-surface
			else
			{
                if(CRM32Pro.XMLAttributeGet(idXML, "source", &pAux) != 1) goto image_err;
				pSurface = IImage->LoadBMP(pAux);
                if(CRM32Pro.XMLAttributeGet(idXML, "colorkey", &iTmp) != 1) goto image_err;
				if(iTmp != -1) SDL_SetColorKey(pSurface, SDL_SRCCOLORKEY | SDL_RLEACCEL, iTmp);
				else { SDL_SetColorKey(pSurface, 0, 0); pSurface->format->colorkey = -1; }
                if(CRM32Pro.XMLAttributeGet(idXML, "alphamod", &iTmp) != 1) goto image_err;
				if(iTmp == 255)	SDL_SetAlpha(pSurface, 0, 255);
				else SDL_SetAlpha(pSurface, SDL_SRCALPHA, iTmp);
			}

			// Save to DPF. Overwrite any previous image with the same name			
			if(IImage->Save(idDPF, pTmp, pSurface) == -1)
			{
				// Remove previous block and try again
                if(RemoveBlock(idDPF, "IMG", pTmp))
                {
                    if(IImage->Save(idDPF, pTmp, pSurface) > 0) bFlagErr = 0;
                }
			}
            else bFlagErr = 0;

        image_err:
            if(pSurface != NULL) SDL_FreeSurface(pSurface);
            if(bFlagErr != 0) printf("\n - WARNING: problem found importing '%s' image.\n", pTmp);
            else
            {
                imageBlocks++;
                iImport++;
            }
		}

		// ----------------------------Importing tiles----------------------------
		else if(!strncmp("tile", pNodeName, 4))
		{
            bFlagErr = 1; // Assume error
            pTmp = "no name attribute";
            pTile = NULL;

            // Get image name and blending mode
            if(CRM32Pro.XMLAttributeGet(idXML, "name", &pTmp) != 1) goto tile_err;
            if(CRM32Pro.XMLAttributeGet(idXML, "alphablending", &pAux) != 1) goto tile_err;

			// Create a new tile
            pTile = new CRM32Pro_CTile;

			// Alpha per-pixel?
			if(!strncmp("per-pixel", pAux, 9))
			{
                if(CRM32Pro.XMLAttributeGet(idXML, "source", &pAux) != 1) goto tile_err;
				pSurface = IImage->LoadPNG(pAux);
                pTile->Create(pSurface, "temp");
				SDL_FreeSurface(pSurface);
			}
			// Alpha per-surface
			else
			{
				if(CRM32Pro.XMLAttributeGet(idXML, "source", &pAux) != 1) goto tile_err;
                pTile->Create(pAux, "temp");
				if(CRM32Pro.XMLAttributeGet(idXML, "colorkey", &iTmp) != 1) goto tile_err;
                pTile->SetColorKey(iTmp);
				if(CRM32Pro.XMLAttributeGet(idXML, "alphamod", &iTmp) != 1) goto tile_err;
                pTile->SetAlpha(iTmp);
			}
			// Positions
			if(CRM32Pro.XMLAttributeGet(idXML, "xposition", &iTmp) != 1) goto tile_err;
			if(CRM32Pro.XMLAttributeGet(idXML, "yposition", &iTmp2) != 1) goto tile_err;
            pTile->SetPosition(iTmp, iTmp2);

			// Tileset?
			if(CRM32Pro.XMLAttributeGet(idXML, "tileset", &pAux) != 1) goto tile_err;
			if(!strncmp("enabled", pAux, 7))
			{
				if(CRM32Pro.XMLAttributeGet(idXML, "xoffset", &iTmp) != 1) goto tile_err;
				if(CRM32Pro.XMLAttributeGet(idXML, "yoffset", &iTmp2) != 1) goto tile_err;
                pTile->SetTileSet(1, iTmp, iTmp2);
			}

			// Save to DPF. Overwrite any previous tile with the same name
            pTile->SetName(pTmp);
			if(pTile->Save(idDPF) == -1)
			{
				// Remove previous block and try again
                if(RemoveBlock(idDPF, "TILE2", pTmp))
                {
                    if(pTile->Save(idDPF) > 0) bFlagErr = 0;
                }
			}
            else bFlagErr = 0;

        tile_err:
            if(pTile != NULL) delete pTile;
            if(bFlagErr != 0) printf("\n - WARNING: problem found importing '%s' tile.\n", pTmp);
            else
            {
                tileBlocks++;
                iImport++;
            }
		}

		// ----------------------------Importing fonts----------------------------
		else if(!strncmp("font", pNodeName, 4))
		{
            bFlagErr = 1; // Assume error
            pTmp = "no name attribute";
            pFont = NULL;

            // Get font name and blending mode
            if(CRM32Pro.XMLAttributeGet(idXML, "name", &pTmp) != 1) goto font_err;
            if(CRM32Pro.XMLAttributeGet(idXML, "alphablending", &pAux) != 1) goto font_err;

			// Create a new font
            pFont = new CRM32Pro_CFont;

			// Alpha per-pixel?
			if(!strncmp("per-pixel", pAux, 9))
			{
				if(CRM32Pro.XMLAttributeGet(idXML, "source", &pAux) != 1) goto font_err;
				pSurface = IImage->LoadPNG(pAux);
                pFont->Create(pSurface);
                SDL_FreeSurface(pSurface);
			}
			// Alpha per-surface
			else
			{
				if(CRM32Pro.XMLAttributeGet(idXML, "source", &pAux) != 1) goto font_err;
				pSurface = IImage->LoadBMP(pAux);
                pFont->Create(pSurface);
                SDL_FreeSurface(pSurface);
				if(CRM32Pro.XMLAttributeGet(idXML, "alphamod", &iTmp) != 1) goto font_err;
                pFont->SetAlpha(iTmp);
			}

			// Kerning
			if(CRM32Pro.XMLAttributeGet(idXML, "kerning", &iTmp) != 1) goto font_err;
            pFont->SetKerning(iTmp);

            // Save to DPF. Overwrite any previous font with the same name
            pFont->SetName(pTmp);
            if(pFont->Save(idDPF, pTmp) == -1)
            {
                // Remove previous block and try again
                if(RemoveBlock(idDPF, "FONT2", pTmp))
                {
                    if(pFont->Save(idDPF, pTmp) > 0) bFlagErr = 0;
                }
            }
            else bFlagErr = 0;

        font_err:
            if(pFont != NULL) delete pFont;
            if(bFlagErr != 0) printf("\n - WARNING: problem found importing '%s' font.\n", pTmp);
            else
            {
                fontBlocks++;
                iImport++;
            }
		}

		// ----------------------------Importing sprites----------------------------
		else if(!strncmp("sprite", pNodeName, 6))
		{
            bFlagErr = 1; // Assume error
            pTmp = "no name attribute";
            pSprite = NULL;
            int iParsedAnimations = 0;

            // Get sprite name and blending mode
            if(CRM32Pro.XMLAttributeGet(idXML, "name", &pTmp) != 1) goto sprite_err;
            if(CRM32Pro.XMLAttributeGet(idXML, "alphablending", &pAux) != 1) goto sprite_err;

			// Create a new sprite
            pSprite = new CRM32Pro_CSprite;

			// X/Y offsets
			if(CRM32Pro.XMLAttributeGet(idXML, "xoffset", &iTmp) != 1) goto sprite_err;
			if(CRM32Pro.XMLAttributeGet(idXML, "yoffset", &iTmp2) != 1) goto sprite_err;

			// Alpha per-pixel?
			if(!strncmp("per-pixel", pAux, 9))
			{
				if(CRM32Pro.XMLAttributeGet(idXML, "source", &pAux) != 1) goto sprite_err;
				pSurface = IImage->LoadPNG(pAux);
				pSprite->Create(pSurface, "temp", iTmp, iTmp2);
				SDL_FreeSurface(pSurface);
			}
			// Alpha per-surfaced
			else
			{
				if(CRM32Pro.XMLAttributeGet(idXML, "source", &pAux) != 1) goto sprite_err;
				pSprite->Create(pAux, "temp", iTmp, iTmp2);
				if(CRM32Pro.XMLAttributeGet(idXML, "colorkey", &iTmp) != 1) goto sprite_err;
				pSprite->SetColorKey(iTmp);
				if(CRM32Pro.XMLAttributeGet(idXML, "alphamod", &iTmp) != 1) goto sprite_err;
				pSprite->SetAlpha(iTmp);
			}
            int iAnimations = pSprite->GetSurface()->h / iTmp2;

			// Loop through each animation
			retvalue = CRM32Pro.XMLNodeChild(idXML); // Point to the first animation on the XML
			while(retvalue != 0)
			{
                ++iParsedAnimations;
                pAux = CRM32Pro.XMLNodeGetName(idXML);
                if(strncmp(pAux, "animation", 9))
                {
                    printf("\n - WARNING: no 'animation' tag found.");
                    goto sprite_err;
                }
				
				int idAnim = 0, type = 0, status = 0, frames = 0, startframe = 0, endframe = 0;
				int hotx = 0, hoty = 0;
				if(CRM32Pro.XMLAttributeGet(idXML, "id", &idAnim) != 1) goto sprite_err;
				if(CRM32Pro.XMLAttributeGet(idXML, "type", &type) != 1) goto sprite_err;
				if(CRM32Pro.XMLAttributeGet(idXML, "status", &status) != 1) goto sprite_err;
				if(CRM32Pro.XMLAttributeGet(idXML, "frames", &frames) != 1) goto sprite_err;
				if(CRM32Pro.XMLAttributeGet(idXML, "hotx", &hotx) != 1) goto sprite_err;
				if(CRM32Pro.XMLAttributeGet(idXML, "hoty", &hoty) != 1) goto sprite_err;
				if(CRM32Pro.XMLAttributeGet(idXML, "starframe", &startframe) != 1) goto sprite_err;
				if(CRM32Pro.XMLAttributeGet(idXML, "endframe", &endframe) != 1) goto sprite_err;
				if(idAnim < 0 || idAnim > iAnimations)
				{
                    printf("\n - WARNING: idAnim '%d' is out of range [0,%d].", idAnim, iAnimations);
                    goto sprite_err;
				}
                if((type != SPRANIM_LOOP) && (type != SPRANIM_PINGPONG) && (type != SPRANIM_ONESHOT))
                {
                    printf("\n - WARNING: type '%d' is unknown.", type);
                    goto sprite_err;
                }
                pSprite->SetAnimProperties(idAnim + 1, type, status, hotx, hoty, startframe, endframe);

                // Loop through frames on current animation
                retvalue = CRM32Pro.XMLNodeChild(idXML); // Point to the first frame on the XML
                if(retvalue != 0)
                {
                    int j;
                    for(j = 0; (j < frames && retvalue != 0); ++j)
                    {
                        int ms = 0, idFrame;
                        pAux = CRM32Pro.XMLNodeGetName(idXML);
                        if(strncmp(pAux, "frame", 5))
                        {
                            CRM32Pro.XMLNodeParent(idXML); // Point to parent node (animation)
                            printf("\n - WARNING: idAnim '%d': no 'frame' tag found.", idAnim);
                            goto sprite_err;
                        }

                        if(CRM32Pro.XMLAttributeGet(idXML, "id", &idFrame) != 1)
                        {
                            CRM32Pro.XMLNodeParent(idXML); // Point to parent node (animation)
                            printf("\n - WARNING: idAnim '%d': no 'id' frame attribute found.", idAnim);
                            goto sprite_err;
                        }
                        if(CRM32Pro.XMLAttributeGet(idXML, "ms", &ms) != 1)
                        {
                            CRM32Pro.XMLNodeParent(idXML); // Point to parent node (animation)
                            printf("\n - WARNING: idAnim '%d': no 'ms' frame attribute found.", idAnim);
                            goto sprite_err;
                        }
                        if(idFrame < 0 || idFrame > frames)
                        {
                            CRM32Pro.XMLNodeParent(idXML); // Point to parent node (animation)
                            printf("\n - WARNING: idAnim '%d': idFrame '%d' is out of range [0,%d].", idAnim, idFrame, frames);
                            goto sprite_err;
                        }
                        pSprite->SetFrameProperties(idAnim + 1, idFrame + 1, ms);
                        
                        // Next frame
                        retvalue = CRM32Pro.XMLNodeNext(idXML);                        
                    }                    
                    CRM32Pro.XMLNodeParent(idXML); // Point to parent node (animation)
                    // Check if we parsed all frames
                    if(frames != j)
                    {
                        printf("\n - WARNING: idAnim '%d': parsed '%d' frames from a total of '%d'.", idAnim, j + 1, frames);
                        goto sprite_err;
                    }
                }

				// Try next animation
				retvalue = CRM32Pro.XMLNodeNext(idXML);
			}

            // Check if we parsed all the animations
            if(iAnimations != iParsedAnimations)
            {
                printf("\n - WARNING: parsed '%d' animations from a total of '%d'.", iParsedAnimations, iAnimations);
                goto sprite_err;
            }

            // Save to DPF. Overwrite any previous sprite with the same name
            pSprite->SetName(pTmp);
            if(pSprite->Save(idDPF) == -1)
            {
                // Remove previous block and try again
                if(RemoveBlock(idDPF, "SPR2", pTmp))
                {
                    if(pSprite->Save(idDPF) > 0) bFlagErr = 0;
                }
            }
            else bFlagErr = 0;

        sprite_err:
            if(iParsedAnimations > 0) CRM32Pro.XMLNodeParent(idXML);
            if(pSprite != NULL) delete pSprite;
            if(bFlagErr != 0) printf("\n - WARNING: problem found importing '%s' sprite.\n", pTmp);
            else
            {
                spriteBlocks++;
                iImport++;
            }
		}

        // ----------------------------Importing music----------------------------
        else if(!strncmp("music", pNodeName, 5))
        {
            bFlagErr = 1; // Assume error
            pTmp = "no name attribute";

            // Get source music, name and import
            if(CRM32Pro.XMLAttributeGet(idXML, "name", &pTmp) != 1) goto music_err;
            if(CRM32Pro.XMLAttributeGet(idXML, "source", &pAux) != 1) goto music_err;
            if(ISoundFX->MusicImport(idDPF, pTmp, pAux) > 0) bFlagErr = 0;

        music_err:
            if(bFlagErr != 0) printf("\n - WARNING: problem found importing '%s' music.\n", pTmp);
            else
            {
                musicBlocks++;
                iImport++;
            }
        }

        // ----------------------------Importing sound----------------------------
        else if(!strncmp("sound", pNodeName, 5))
        {
            bFlagErr = 1; // Assume error
            pTmp = "no name attribute";

            // Get source sound, name and import
            if(CRM32Pro.XMLAttributeGet(idXML, "name", &pTmp) != 1) goto sound_err;
            if(CRM32Pro.XMLAttributeGet(idXML, "source", &pAux) != 1) goto sound_err;
            if(ISoundFX->SoundImport(idDPF, pTmp, pAux) > 0) bFlagErr = 0;

        sound_err:
            if(bFlagErr != 0) printf("\n - WARNING: problem found importing '%s' sound.\n", pTmp);
            else
            {
                soundBlocks++;
                iImport++;
            }
        }

        // ----------------------------Importing file----------------------------
        else if(!strncmp("file", pNodeName, 4))
        {
            DPF_BlockData myBlock;
            memset((void*)&myBlock, 0, sizeof(myBlock));
            bFlagErr = 1; // Assume error
            pTmp = "no name attribute";

            // Get source file and name
            if(CRM32Pro.XMLAttributeGet(idXML, "name", &pTmp) != 1) goto file_err;
            myBlock.storedmethod = 1;
            if(CRM32Pro.XMLAttributeGet(idXML, "storemethod", &pAux) != 1) goto file_err;
            if(!strcmp(pAux, "compressed")) myBlock.storedmethod = 0;
            else if(!strcmp(pAux, "crypted")) myBlock.storedmethod = 2;
            if(CRM32Pro.XMLAttributeGet(idXML, "source", &pAux) != 1) goto file_err;
            strcpy(myBlock.type, "FILE");
            strcpy(myBlock.name, pTmp);

            // Read file
            int iFile = _open(pAux, O_RDONLY | O_BINARY, 0);
            pAux = NULL;
            if(iFile != -1)
            {   
                int iSize = _filelength(iFile);
                pAux = (char*)malloc(iSize);
                _read(iFile, pAux, iSize);
                myBlock.size = iSize;
                myBlock.data = pAux;
                _close(iFile);
            }

            // Try to add
            if(pAux != NULL)
            {
                if(IStuffDPF->AddData(idDPF, &myBlock) > 0) bFlagErr = 0;
                free(pAux);
            }

        file_err:
            if(bFlagErr != 0) printf("\n - WARNING: problem found importing '%s' file.\n", pTmp);
            else
            {
                fileBlocks++;
                iImport++;
            }
        }

        // ----------------------------Importing widgets----------------------------
        else if(!strncmp("panel", pNodeName, 5))
        {
            // We dont check the panel name as on C32, they are not supported and all widgets are grouped into a single panel    
            int iParsedWidgets = 0; // control when we must return to a parent
            // Loop through all widgets
            retvalue = CRM32Pro.XMLNodeChild(idXML); // Point to the first widget on the XML
            while(retvalue != 0)
            {
                IButtonNew myWidget;
                bFlagErr = 1; // Assume error
                ++iParsedWidgets;

                pTmp = CRM32Pro.XMLNodeGetName(idXML);
                if(strncmp(pTmp, "widget", 6))
                {
                    printf("\n - WARNING: no 'widget' tag found.");
                    goto widget_err;
                }
                if(CRM32Pro.XMLAttributeGet(idXML, "name", &pTmp) != 1) goto widget_err;
                if(CRM32Pro.XMLAttributeGet(idXML, "xposition", &iTmp) != 1) goto widget_err;
                myWidget.posx = (Uint16)iTmp;
                if(CRM32Pro.XMLAttributeGet(idXML, "yposition", &iTmp) != 1) goto widget_err;
                myWidget.posy = (Uint16)iTmp;
                if(CRM32Pro.XMLAttributeGet(idXML, "type", &pAux) != 1) goto widget_err;
                if(!strcmp(pAux, "button")) myWidget.type = B_TYPE_NORMAL;
                else if(!strcmp(pAux, "hslider")) myWidget.type = B_TYPE_SLIDER;
                else if(!strcmp(pAux, "progressbar")) myWidget.type = B_TYPE_PROGRESS;
                else if(!strcmp(pAux, "checkbox")) myWidget.type = B_TYPE_CHECK;
                else if(!strcmp(pAux, "inputbox")) myWidget.type = B_TYPE_INPUT;
                else goto widget_err;
                if(CRM32Pro.XMLAttributeGet(idXML, "key", &iTmp) != 1) goto widget_err;
                myWidget.key = (Uint16)iTmp;
                if(CRM32Pro.XMLAttributeGet(idXML, "selectboxRGB", &iTmp) != 1) goto widget_err;
                myWidget.color_select = (Uint32)iTmp;
                if(CRM32Pro.XMLAttributeGet(idXML, "value", &iTmp) != 1) goto widget_err;
                myWidget.cvalue = iTmp;
                if(CRM32Pro.XMLAttributeGet(idXML, "range", &iTmp) != 1) goto widget_err;
                myWidget.range = iTmp;
                
                // Loop through widget states represented by an 'image' tag
                retvalue = CRM32Pro.XMLNodeChild(idXML);
                if(retvalue != 0)
                {
                    while(retvalue != 0)
                    {
                        pAux = CRM32Pro.XMLNodeGetName(idXML);
                        if(strncmp(pAux, "image", 5))
                        {
                            CRM32Pro.XMLNodeParent(idXML); // Point to parent node (widget)
                            printf("\n - WARNING: no 'image' tag found.");
                            goto widget_err;
                        }

                        if(CRM32Pro.XMLAttributeGet(idXML, "state", &pAux) != 1)
                        {
                            CRM32Pro.XMLNodeParent(idXML); // Point to parent node (widget)
                            printf("\n - WARNING: no 'state' attribute found.");
                            goto widget_err;
                        }

                        // Detect complex button
                        if(!strcmp(pAux, "hovered") && myWidget.type == B_TYPE_NORMAL) myWidget.type = B_TYPE_COMPLEX;

                        // Load widget state. 
                        // Note: if the loading of a state fails, previous one could be stored on the DPF although will be removed on next attempt
                        if(widgetLoadState(idDPF, idXML, pTmp, pAux)) iImport++;
                        else
                        {
                            CRM32Pro.XMLNodeParent(idXML); // Point to parent node (widget)
                            printf("\n - WARNING: could not load '%s' state.", pAux);
                            goto widget_err;
                        }

                        // Next state
                        retvalue = CRM32Pro.XMLNodeNext(idXML);
                    }
                    CRM32Pro.XMLNodeParent(idXML); // Point to parent node (widget)
                }

                // Save to DPF. Overwrite any previous button with the same name
                iTmp = IButton->Save(idDPF, pTmp, &myWidget);
                if(iTmp == -1)
                {
                    // Remove previous block and try again
                    if(RemoveBlock(idDPF, "BUT", pTmp))
                    {
                        iTmp = IButton->Save(idDPF, pTmp, &myWidget);
                    }
                }
                if(iTmp > 0) bFlagErr = 0;
                else
                {
                    printf("\n - WARNING: widget could not be saved (ErrCode '%d').", iTmp);
                }

                // Try next widget and report failures
            widget_err:
                retvalue = CRM32Pro.XMLNodeNext(idXML);
                if(bFlagErr != 0) printf("\n - WARNING: problem found importing '%s' panel, widget '%s'.\n", "default", pTmp);
                else
                {
                    widgetBlocks++;
                    iImport++;
                }
            }
            if(iParsedWidgets > 0) CRM32Pro.XMLNodeParent(idXML);
        }

		// Info & stats
        printf(" - %d/%d: IMG %d - TIL %d - SPR %d - FNT %d - MUS %d - SND %d - FIL %d - GUI %d\r",        
            iImport, totalBlocks, imageBlocks, tileBlocks, spriteBlocks, fontBlocks, musicBlocks, soundBlocks, fileBlocks, widgetBlocks);
		
        // Next item
		retvalue = CRM32Pro.XMLNodeNext(idXML);		
	}

	// WARNING message is there were blocks not imported
	if(totalBlocks != iImport) printf("\n - WARNING: existing XML blocks(%d) does not match with imported ones(%d).\n", totalBlocks, iImport);
	// Close the XML,DPF and return
	CRM32Pro.XMLClose(idXML);
	IStuffDPF->Close(idDPF);
	return 1;
}

int widgetLoadState(int idDPF, int idXML, char* pName, char* pState)
{
    CRM32Pro_CTile* pTile = NULL;
    SDL_Surface* pSurface;
    char* pTmp, szTmp[255];
    int iTmp, iRet = 0;

    // Try to create the tile
    if(CRM32Pro.XMLAttributeGet(idXML, "alphablending", &pTmp) != 1)  goto loadstate_err;
    pTile = new CRM32Pro_CTile;
    // Alpha per-pixel?
    if(!strncmp("per-pixel", pTmp, 9))
    {
        if(CRM32Pro.XMLAttributeGet(idXML, "source", &pTmp) != 1) goto loadstate_err;
        pSurface = IImage->LoadPNG(pTmp);
        pTile->Create(pSurface, "temp");
        SDL_FreeSurface(pSurface);
    }
    // Alpha per-surface
    else
    {
        if(CRM32Pro.XMLAttributeGet(idXML, "source", &pTmp) != 1) goto loadstate_err;
        pTile->Create(pTmp, "temp");
        if(CRM32Pro.XMLAttributeGet(idXML, "colorkey", &iTmp) != 1) goto loadstate_err;
        pTile->SetColorKey(iTmp);
        if(CRM32Pro.XMLAttributeGet(idXML, "alphamod", &iTmp) != 1) goto loadstate_err;
        pTile->SetAlpha(iTmp);
    }
   
    // Generate blockname
    strcpy(szTmp, pName);
    if(!strcmp(pState, "normal")) strcat(szTmp, "_n");
    else if(!strcmp(pState, "hovered")) strcat(szTmp, "_s");
    else if(!strcmp(pState, "pressed")) strcat(szTmp, "_p");
    else if(!strcmp(pState, "deactivated")) strcat(szTmp, "_o");

    // Save to DPF. Overwrite any previous tile with the same name
    pTile->SetName(szTmp);
    if(pTile->Save(idDPF, "TILB2") == -1)
    {
        // Remove previous block and try again
        if(RemoveBlock(idDPF, "TILB2", szTmp))
        {
            if(pTile->Save(idDPF, "TILB2") > 0) iRet = 1;
        }
    }
    else iRet = 1;    

loadstate_err:
    if(pTile != NULL) delete pTile;
    return iRet;
}

// Remove the given DPF/type/name
// Return 0 Failed
//        1 Ok
int RemoveBlock(int idDPF, char *type, char *name)
{
	DPF_BlockData block;
	if(idDPF == -1) return 0;

	strcpy(block.name, name);
	strcpy(block.type, type);
	return IStuffDPF->Delete(idDPF, &block);
}
